@@ -42,6 +42,7 @@
 
 #include <Kirigami/Platform/TabletModeWatcher>
 #include <algorithm>
+#include <knotification.h>
 
 #ifdef Q_OS_LINUX
 #include <sys/timerfd.h>
@@ -291,11 +292,13 @@ void Core::reparseConfiguration()
         m_criticalBatteryTimer->stop();
         if (m_criticalBatteryNotification) {
             m_criticalBatteryNotification->close();
+            m_criticalBatteryNotification = nullptr;
         }
     }
 
     if (m_lowBatteryNotification && currentChargePercent() > m_globalSettings->batteryLowLevel()) {
         m_lowBatteryNotification->close();
+        m_lowBatteryNotification = nullptr;
     }
 
     readChargeThreshold();
@@ -459,11 +462,13 @@ void Core::onDeviceAdded(const QString &udi)
     // higher than the battery critical level. (See bug 329537)
     if (m_lowBatteryNotification && currentChargePercent() > m_globalSettings->batteryLowLevel()) {
         m_lowBatteryNotification->close();
+        m_lowBatteryNotification = nullptr;
     }
 
     if (currentChargePercent() > m_globalSettings->batteryCriticalLevel()) {
         if (m_criticalBatteryNotification) {
             m_criticalBatteryNotification->close();
+            m_criticalBatteryNotification = nullptr;
         }
 
         if (m_criticalBatteryTimer->isActive()) {
@@ -616,6 +621,7 @@ void Core::handleCriticalBattery(int percent)
 {
     if (m_lowBatteryNotification) {
         m_lowBatteryNotification->close();
+        m_lowBatteryNotification = nullptr;
     }
 
     // no parent, but it won't leak, since it will be closed both in case of timeout or direct action
@@ -659,6 +665,7 @@ void Core::handleCriticalBattery(int percent)
     connect(cancelAction, &KNotificationAction::activated, this, [this] {
         m_criticalBatteryTimer->stop();
         m_criticalBatteryNotification->close();
+        m_criticalBatteryNotification = nullptr;
     });
 
     m_criticalBatteryNotification->sendEvent();
@@ -683,25 +690,48 @@ void Core::onAcAdapterStateChanged(BatteryController::AcAdapterState state)
     loadProfile();
 
     if (state == BatteryController::Plugged) {
-        // If the AC Adaptor has been plugged in, let's clear some pending suspend actions
+        // If the AC Adapter has been plugged in, let's clear some other notifications that will no longer be relevant
         if (m_lowBatteryNotification) {
             m_lowBatteryNotification->close();
+            m_lowBatteryNotification = nullptr;
         }
 
         if (m_criticalBatteryNotification) {
             m_criticalBatteryNotification->close();
+            m_criticalBatteryNotification = nullptr;
         }
 
+        if (m_powerCordUnpluggedNotification) {
+            m_powerCordUnpluggedNotification->close();
+            m_powerCordUnpluggedNotification = nullptr;
+        }
+
+        m_powerCordPluggedInNotification = new KNotification(QStringLiteral("pluggedin"));
+        m_powerCordPluggedInNotification->setComponentName(QStringLiteral("powerdevil"));
+        m_powerCordPluggedInNotification->setUrgency(KNotification::LowUrgency);
+
         if (m_criticalBatteryTimer->isActive()) {
             m_criticalBatteryTimer->stop();
-            emitRichNotification(QStringLiteral("pluggedin"), //
-                                 i18n("Power Cord Plugged In"),
-                                 i18n("The computer will no longer go to sleep."));
+            m_powerCordPluggedInNotification->setTitle(i18n("Power Cord Plugged In"));
+            m_powerCordPluggedInNotification->setText(i18n("The computer will no longer go to sleep."));
         } else {
-            emitRichNotification(QStringLiteral("pluggedin"), i18n("Running on AC power"), i18n("The power cord has been plugged in."));
+            m_powerCordPluggedInNotification->setTitle(i18n("Running on AC Power"));
+            m_powerCordPluggedInNotification->setText(i18n("The power cord has been plugged in."));
         }
+        m_powerCordPluggedInNotification->sendEvent();
     } else if (state == BatteryController::Unplugged) {
-        emitRichNotification(QStringLiteral("unplugged"), i18n("Running on Battery Power"), i18n("The power cord has been unplugged."));
+        if (m_powerCordPluggedInNotification) {
+            m_powerCordPluggedInNotification->close();
+            m_powerCordPluggedInNotification = nullptr;
+        }
+
+        m_powerCordUnpluggedNotification = new KNotification(QStringLiteral("unplugged"));
+        m_powerCordUnpluggedNotification->setComponentName(QStringLiteral("powerdevil"));
+        m_powerCordUnpluggedNotification->setUrgency(KNotification::LowUrgency);
+        m_powerCordUnpluggedNotification->setTitle(i18n("Running on Battery Power"));
+        m_powerCordUnpluggedNotification->setText(i18n("The power cord has been unplugged."));
+
+        m_powerCordUnpluggedNotification->sendEvent();
     }
 }
 
@@ -771,6 +801,7 @@ void Core::onCriticalBatteryTimerExpired()
 {
     if (m_criticalBatteryNotification) {
         m_criticalBatteryNotification->close();
+        m_criticalBatteryNotification = nullptr;
     }
 
     // Do that only if we're not on AC
